from django.contrib import admin


class SingletonAdmin(admin.ModelAdmin):
    actions = None

    def __init__(self, model, admin_site):
        fields = [field.name for field in model._meta.fields]
        self.list_display = fields[:5]
        self.list_display_links = fields[:5]
        super().__init__(model, admin_site)

    def has_add_permission(self, request):
        if not self.model.objects.count():
            return True
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class SingletonActiveAdmin(admin.ModelAdmin):
    def __init__(self, model, admin_site):
        fields = [field.name for field in model._meta.fields]
        self.list_display = fields[:5]
        self.list_display_links = fields[:5]
        self.list_filter = ['active']
        super().__init__(model, admin_site)



from django import template
from django.apps import apps

register = template.Library()


@register.simple_tag
def get_singleton(app_label, model_name):
    model = apps.get_model(app_label, model_name)
    return model.get_object()



default_app_config = 'djsingleton.apps.DjsingletonConfig'
__name__ = 'djsingleton'
__author__ = 'Neeraj Kumar'
__version__ = '1.1.0'
__author_email__ = 'sainineeraj1234@gmail.com'
__description__ = 'This is useful for create singleton model in django for make global settings.'



from django.db import models
from django.utils.translation import gettext as _


class SingletonModel(models.Model):
    """
    This model will save only one entry of model
    """

    def save(self, *args, **kwargs):
        self.pk = 1
        super().save(*args, **kwargs)

    @classmethod
    def get_object(cls):
        obj, created = cls.objects.get_or_create(pk=1)
        return obj

    class Meta:
        abstract = True


class SingletonActiveQueryset(models.query.QuerySet):
    def delete(self):
        for obj in self:
            if not obj.active:
                obj.delete()


class SingletonActiveModel(models.Model):
    """
    This model 'active' field value will True for single entry.
     Ex: If you have multiple API keys,
      So you can only activate any key at a time.
    """
    active = models.BooleanField(default=False)
    objects = SingletonActiveQueryset.as_manager()

    def save(self, *args, **kwargs):
        count = self.__class__.objects.count()
        if not count or (count == 1 and self.pk):
            self.active = True
            self.pk = 1
        elif self.active:
            self.__class__.objects.filter(active=True).update(active=False)
        elif not self.active and self.pk:
            self.active = True
        super().save(*args, **kwargs)

    @classmethod
    def get_object(cls):
        obj, created = cls.objects.get_or_create(active=True)
        return obj

    def delete(self, *args, **kwargs):
        if self.active:
            raise Exception(_('Active entry will not delete.'))
        return super().delete(*args, **kwargs)

    class Meta:
        abstract = True



from setuptools import find_packages, setup

import djsingleton

setup(
    name=djsingleton.__name__,
    version=djsingleton.__version__,
    packages=find_packages(),
    url='https://github.com/sainipray/djsingleton',
    author=djsingleton.__author__,
    author_email=djsingleton.__author_email__,
    description=djsingleton.__description__,
    license='MIT',
    include_package_data=True,
    platforms=['any'],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 2',
        'Framework :: Django :: 3',
        'Framework :: Django :: 4',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development',
    ],
)
